var STATE_NORMAL = 'STATE_NORMAL';
var STATE_FLOAT = 'STATE_FLOAT';
var STATE_OPERATION = 'STATE_OPERATION';
var STATE_SOLVED = 'STATE_SOLVED';

var OP_INC = '+';
var OP_DEC = '-';
var OP_MUL = '*';
var OP_DIV = '/';
var OP_EQ = '=';

var OP_POINT = '.';

function Calculator() {
    this.value = '0';
    this.state = STATE_NORMAL;
    this.operation = null;
    this.memory = 0;
}

Calculator.prototype.val = function() {
    return parseFloat(this.value);
};

Calculator.prototype._memoryString = function() {
    var memory = this.memory
        + (Math.floor(this.memory) === this.memory ? '.' : '');

    return memory.length > 10 ? 'E' + memory.substr(0, 9) : memory;
};

Calculator.prototype._valueString = function() {
    return this.value + (this.state === STATE_FLOAT ? '' : '.' );
};

Calculator.prototype.displayValue = function() {
    return (
        ([STATE_SOLVED, STATE_OPERATION].indexOf(this.state) >= 0)
        ? this._memoryString()
        : this._valueString()
    );
};

Calculator.prototype.execute = function() {
    switch(this.operation) {
        case OP_INC:
            this.memory += this.val();
            break;
        case OP_DEC:
            this.memory -= this.val();
            break;
        case OP_MUL:
            this.memory *= this.val();
            break;
        case OP_DIV:
            this.memory /= this.val();
            break;
        default:
            this.memory = this.val();
    }

    this.value = '0';
};

Calculator.prototype.nextStep = function(operation) {
    if(this.state !== STATE_OPERATION) {
        this.execute();
    }

    this.operation = operation;
    this.state = (operation === OP_EQ ? STATE_SOLVED : STATE_OPERATION);
};

Calculator.prototype._nullableState = function(digit) {
    return (
        digit === '0'
            && (this.state === STATE_OPERATION || this.value === '0')
        ) || (digit !== '0' && this.value === '0');
};

Calculator.prototype.addDigit = function(digit) {
    if(this.value.length >= 8) {
        return;
    }

    if(this.state === STATE_SOLVED) {
        this.memory = 0;
    }

    if(this.state !== STATE_FLOAT) {
        if (digit === OP_POINT) {
            this.state = STATE_FLOAT;

        } else {
            this.state = STATE_NORMAL;
        }

        if(this._nullableState(digit)) {
            this.value = '';
        }
    }

    this.value += digit;
};
